<?php

namespace Drupal\legal\EventSubscriber;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Session\Flash\FlashBagInterface;
use Symfony\Component\HttpFoundation\Session\SessionInterface;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Prevent users from viewing the site until they accept the T&C.
 */
class LegalNavigationLock implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * Account proxy.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $account;

  /**
   * Flash bag.
   *
   * @var \Symfony\Component\HttpFoundation\Session\Flash\FlashBagInterface
   */
  protected $flashBag;

  /**
   * Messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The current session.
   *
   * @var \Symfony\Component\HttpFoundation\Session\SessionInterface
   */
  protected $session;

  /**
   * Current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Constructs the Legal navigation lock event subscriber.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $account
   *   Account proxy.
   * @param \Symfony\Component\HttpFoundation\Session\Flash\FlashBagInterface $flash_bag
   *   Flash bag.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   Messenger.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   Module handler.
   * @param \Symfony\Component\HttpFoundation\Session\SessionInterface $session
   *   The current session.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The current route match.
   */
  public function __construct(AccountProxyInterface $account, FlashBagInterface $flash_bag, MessengerInterface $messenger, ModuleHandlerInterface $module_handler, SessionInterface $session, RouteMatchInterface $route_match) {
    $this->account = $account;
    $this->flashBag = $flash_bag;
    $this->messenger = $messenger;
    $this->moduleHandler = $module_handler;
    $this->session = $session;
    $this->routeMatch = $route_match;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[KernelEvents::REQUEST][] = ['onKernelRequestLockNavigation', 31];
    return $events;
  }

  /**
   * Locks the user to the legal_accept from to accept the T&C.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   Response event.
   */
  public function onKernelRequestLockNavigation(RequestEvent $event): void {

    if ($this->account->isAnonymous()) {
      // Do not nav-lock anonymous users, including blocked users who just tried
      // to log into the site.
      return;
    }

    if ($this->session->get('legal.accept_form_lock', NULL)) {
      // If the user hasn't accepted the legal yet, force to stay on the
      // legal_accept page.
      $allowed_paths = [];
      $allowed_routes = [
        'legal.legal',
        'legal.legal_login',
        'system.css_asset',
        'system.js_asset',
        'user.logout',
      ];

      // Allow altering the allowed paths.
      $this->moduleHandler->alter('legal_allowed_paths', $allowed_paths);
      // Allow altering the allowed routes.
      $this->moduleHandler->alter('legal_allowed_routes', $allowed_routes);

      $current_route_name = $this->routeMatch->getRouteName();
      $current_path = $event->getRequest()->getPathInfo();
      $current_query = $event->getRequest()->getQueryString();

      // Unset password reset message on the legal page.
      if ('legal.legal_login' === $current_route_name) {
        $status_messages = $this->flashBag->get('status');
        foreach ($status_messages as $key => $message) {
          if ((string) $message == (string) $this->t('You have just used your one-time login link. It is no longer necessary to use this link to log in. Please change your password.')) {
            unset($status_messages[$key]);
            if (!empty($status_messages)) {
              $this->flashBag->set('status', $status_messages);
            }
            break;
          }
        }
      }

      if (!in_array($current_path, $allowed_paths) && !in_array($current_route_name, $allowed_routes)) {
        // Make sure that the error message is not being set on the initial
        // redirect when the user logs in.
        if ($this->session->has('legal.initial_redirect')) {
          $this->session->remove('legal.initial_redirect');
          if ($current_path !== '/') {
            $saved_path = $current_query ? $current_path . '?' . $current_query : $current_path;
            $this->session->set('legal.original_destination', $saved_path);
          }
        }
        else {
          $this->messenger->addError($this->t('You must accept the Terms and Conditions before continuing!'));
        }

        $response = new RedirectResponse(URL::fromRoute('legal.legal_login')->toString());
        $event->setResponse($response);
      }
    }
  }

}
