<?php

namespace Drupal\legal\Form;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\legal\Entity\Accepted;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * After login display new T&Cs to user and require that they are agreed to.
 *
 * User has been logged out before arriving at this page,
 * and is logged back in if they accept T&Cs.
 */
class LegalLogin extends FormBase {

  /**
   * Language handling.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * User storage.
   *
   * @var \Drupal\user\UserStorageInterface
   */
  protected $userStorage;

  /**
   * The cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The current session.
   *
   * @var \Symfony\Component\HttpFoundation\Session\SessionInterface
   */
  protected $session;

  /**
   * Page cache kill switch.
   *
   * @var \Drupal\Core\PageCache\ResponsePolicy\KillSwitch
   */
  protected $pageCacheKillSwitch;

  /**
   * Constructs a new LegalLogin object.
   *
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack service.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache backend.
   * @param \Drupal\Core\PageCache\ResponsePolicy\KillSwitch $page_cache_kill_switch
   *   The page cache kill switch service.
   */
  public function __construct(LanguageManagerInterface $language_manager, EntityTypeManagerInterface $entity_type_manager, RequestStack $request_stack, CacheBackendInterface $cache, KillSwitch $page_cache_kill_switch) {
    $this->languageManager = $language_manager;
    $this->userStorage = $entity_type_manager->getStorage('user');
    $this->session = $request_stack->getCurrentRequest()->getSession();
    $this->cache = $cache;
    $this->pageCacheKillSwitch = $page_cache_kill_switch;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('language_manager'),
      $container->get('entity_type.manager'),
      $container->get('request_stack'),
      $container->get('cache.menu'),
      $container->get('page_cache_kill_switch'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'legal_login';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config   = $this->config('legal.settings');
    $language = $this->languageManager->getCurrentLanguage();

    // Get last accepted version for this account.
    $uid = $this->currentUser()->id();
    $legal_account = legal_get_accept($uid);

    // If no version accepted, get version with current language revision.
    if (empty($legal_account['version'])) {
      $conditions = legal_get_conditions($language->getId());
      // No conditions set yet.
      if (empty($conditions['conditions'])) {
        return [];
      }
    }
    else {
      // Get version / revision of last accepted language.
      $conditions = legal_get_conditions($legal_account['language']);
      // No conditions set yet.
      if (empty($conditions['conditions'])) {
        return [];
      }
      // Check latest version of T&C has been accepted.
      $accepted = legal_version_check($uid, $conditions['version'], $conditions['revision'], $legal_account);

      if ($accepted && $config->get('accept_every_login') == 0) {
        return [];
      }
    }

    legal_display_fields($form, $conditions, 'login');

    $form['uid'] = [
      '#type'  => 'value',
      '#value' => $uid,
    ];

    $form['tc_id'] = [
      '#type'  => 'value',
      '#value' => $conditions['tc_id'],
    ];

    $form['version'] = [
      '#type'  => 'value',
      '#value' => $conditions['version'],
    ];

    $form['revision'] = [
      '#type'  => 'value',
      '#value' => $conditions['revision'],
    ];

    $form['language'] = [
      '#type'  => 'value',
      '#value' => $conditions['language'],
    ];

    $form = legal_display_changes($form, $uid);

    $form['save'] = [
      '#type'   => 'submit',
      '#value'  => $this->t('Confirm'),
      '#weight' => 100,
    ];

    // @todo Is it really necessary?
    // Prevent this page from being cached.
    $this->pageCacheKillSwitch->trigger();

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    // Get form values.
    $values = $form_state->getValues();

    // Set the redirect URL.
    $form_state->setRedirectUrl($this->getRedirectUrl());

    // Option to require user to accept T&Cs on every login.
    $config = $this->config('legal.settings');
    if ($config->get('accept_every_login') == '1') {
      // Get last accepted version for this account.
      $legal_account = legal_get_accept($values['uid']);
      $already_accepted = legal_version_check($values['uid'], $values['version'], $values['revision'], $legal_account);

      // If already accepted just update the time.
      if ($already_accepted) {
        $accepted = Accepted::load($legal_account['legal_id']);
        $accepted->set("accepted", time());
        $accepted->save();
      }
      else {
        legal_save_accept($values['version'], $values['revision'], $values['language'], $values['uid']);
      }
    }
    else {
      legal_save_accept($values['version'], $values['revision'], $values['language'], $values['uid']);
    }

    /** @var \Drupal\user\UserInterface $user */
    $user = $this->userStorage->load($this->currentUser()->id());
    $this->logger('legal')
      ->notice('%name accepted T&C version %tc_id.', [
        '%name'  => $user->get('name')->getString(),
        '%tc_id' => $values['tc_id'],
      ]);

    // Clean-up legal session.
    $this->session->remove('legal.accept_form_lock');
    $this->session->remove('legal.initial_redirect');
    $this->session->remove('legal.original_destination');
  }

  /**
   * Return redirect URL.
   *
   * @return \Drupal\Core\Url
   *   The redirect URL
   */
  protected function getRedirectUrl(): Url {
    // Determine redirect URL.
    $default_dest = $this->config('legal.settings')->get('login_redirect_url');
    if (empty($default_dest)) {
      $default_dest = '/user/';
    }

    if ($this->session->has('legal.original_destination')) {
      $destination = $this->session->get('legal.original_destination');
    }
    else {
      $destination = filter_input(INPUT_GET, 'destination', FILTER_UNSAFE_RAW);
    }
    return Url::fromUserInput(!empty($destination) ? $destination : $default_dest);
  }

  /**
   * Access control callback.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Run access checks for this account.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   Returns whether the user has access or not.
   */
  public function access(AccountInterface $account): AccessResultInterface {
    // Allow access to this form only if the accept form lock is active.
    if ($this->session->get('legal.accept_form_lock', NULL)) {
      return AccessResult::allowed();
    }

    return AccessResult::neutral();
  }

}
